@@ -79,9 +79,9 @@ class Check(object):
         if os.path.exists('/etc/ldap.secret'):
             self.lo = univention.uldap.getAdminConnection()
         elif os.path.exists('/etc/machine.secret'):
-            self.lo = univention.uldap.getMachineConnection(start_tls=2)
+            self.lo = univention.uldap.getMachineConnection()
         else:
-            self.lo = univention.uldap.access(host=self.ConfigRegistry.get('ldap/master'), base=self.ConfigRegistry.get('ldap/base'), start_tls=2)
+            self.lo = univention.uldap.access(host=self.ConfigRegistry.get('ldap/master'), base=self.ConfigRegistry.get('ldap/base'))
 
     def _systemPolicy(self):
         if self.ConfigRegistry.get('password/quality/credit/digits', '0'):
