@@ -42,12 +42,17 @@ import ldap
 import ldap.sasl
 import ldap.schema
 import six
+from ldap.controls.readentry import PostReadControl, PreReadControl
 from ldapurl import LDAPUrl, isLDAPUrl
 
 import univention.logging  # noqa: F401
 from univention.config_registry import ConfigRegistry
 
 
+configRegistry = ConfigRegistry()
+configRegistry.load()
+feature_full_prepostread = configRegistry.is_true('directory/manager/feature/prepostread', False)
+
 try:
     from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union  # noqa: F401
 except ImportError:
@@ -712,10 +717,19 @@ class access(object):
         :type serverctrls: list[ldap.controls.LDAPControl]
         :param dict response: An optional dictionary to receive the server controls of the result.
         """
-        if not serverctrls:
+        log.debug('uldap.add dn=%s', dn)
+
+        if feature_full_prepostread:
+            if serverctrls:
+                for ctrl in serverctrls:
+                    if isinstance(ctrl, PostReadControl):
+                        log.info('uldap.add: overriding PostReadControl (%s)', ctrl.attrList)
+                        ctrl.attrList = ['*', '+']
+            else:
+                serverctrls = [PostReadControl(True, ['*', '+'])]
+        elif not serverctrls:
             serverctrls = []
 
-        log.debug('uldap.add dn=%s', dn)
         nal = {}  # type: Dict[str, Any]
         for i in al:
             key, val = i[0], i[-1]
@@ -755,7 +769,16 @@ class access(object):
         """
         log.debug('uldap.modify %s', dn)
 
-        if not serverctrls:
+        if feature_full_prepostread:
+            if serverctrls:
+                for ctrl in serverctrls:
+                    for ctrl_type in (PreReadControl, PostReadControl):
+                        if isinstance(ctrl, ctrl_type):
+                            log.info('uldap.modify: overriding %s (%s)', type(ctrl_type).__name__, ctrl.attrList)
+                            ctrl.attrList = ['*', '+']
+            else:
+                serverctrls = [PreReadControl(True, ['*', '+']), PostReadControl(True, ['*', '+'])]
+        elif not serverctrls:
             serverctrls = []
 
         ml = []
@@ -875,7 +898,16 @@ class access(object):
         newrdn = ldap.dn.dn2str([ldap.dn.str2dn(newdn)[0]])
         newsdn = ldap.dn.dn2str(ldap.dn.str2dn(newdn)[1:])
 
-        if not serverctrls:
+        if feature_full_prepostread:
+            if serverctrls:
+                for ctrl in serverctrls:
+                    for ctrl_type in (PreReadControl, PostReadControl):
+                        if isinstance(ctrl, ctrl_type):
+                            log.info('uldap.rename: overriding %s (%s)', type(ctrl_type).__name__, ctrl.attrList)
+                            ctrl.attrList = ['*', '+']
+            else:
+                serverctrls = [PreReadControl(True, ['*', '+']), PostReadControl(True, ['*', '+'])]
+        elif not serverctrls:
             serverctrls = []
 
         if oldsdn and newsdn.lower() == oldsdn.lower():
@@ -913,23 +945,41 @@ class access(object):
             response['ctrls'] = resp_ctrls
 
     @_fix_reconnect_handling
-    def delete(self, dn):
+    def delete(self, dn, serverctrls=None, response=None):
         # type: (str) -> None
         """
         Delete a LDAP object.
 
         :param str dn: The distinguished name of the object to remove.
+        :param serverctrls: a list of :py:class:`ldap.controls.LDAPControl` instances sent to the server along with the LDAP request.
+        :type serverctrls: list[ldap.controls.LDAPControl]
+        :param dict response: An optional dictionary to receive the server controls of the result.
         """
         log.debug('uldap.delete %s', dn)
+
+        if feature_full_prepostread:
+            if serverctrls:
+                for ctrl in serverctrls:
+                    if isinstance(ctrl, PreReadControl):
+                        log.info('uldap.delete: overriding PreReadControl (%s)', ctrl.attrList)
+                        ctrl.attrList = ['*', '+']
+            else:
+                serverctrls = [PreReadControl(True, ['*', '+'])]
+        elif not serverctrls:
+            serverctrls = []
+
         if dn:
             log.debug('delete')
             try:
-                self.lo.delete_s(dn)
+                _rtype, _rdata, _rmsgid, resp_ctrls = self.lo.delete_ext_s(dn, serverctrls=serverctrls)
             except ldap.REFERRAL as exc:
                 if not self.follow_referral:
                     raise
                 lo_ref = self._handle_referral(exc)
-                lo_ref.delete_s(dn)
+                _rtype, _rdata, _rmsgid, resp_ctrls = lo_ref.delete_ext_s(dn, serverctrls=serverctrls)
+
+        if serverctrls and isinstance(response, dict):
+            response['ctrls'] = resp_ctrls
 
     def parentDn(self, dn):
         # type: (str) -> Optional[str]
